<?php
/**
 * $Horde: accounts/lib/Driver.php,v 1.16 2005/01/03 14:34:44 jan Exp $
 *
 * Copyright 2001-2005 Eric Rostetter <eric.rostetter@physics.utexas.edu>
 *
 * See the enclosed file LICENSE for license information (BSD). If you
 * did not receive this file, see http://www.horde.org/licenses/bsdl.php.
 *
 * Accounts_Driver:: defines an API for getting/displaying account information
 * for a user for the accounts module.
 *
 * @author   Eric Rostetter <eric.rostetter@physics.utexas.edu>
 * @version  $Revision: 1.16 $
 * @since    Accounts 2.1
 * @package  accounts
 */
class Accounts_Driver {

    /**
     * Error string returned to user if an error occurs.
     *
     * @var string $err_str
     */
    var $err_str;

    /**
     * Attempts to return a concrete Accounts_Driver instance based on $driver.
     *
     * @param string $driver  The type of concrete Accounts_Driver subclass to
     *                        return. The class is based on the accounts driver
     *                        ($driver). The code is dynamically included.
     *
     * @param array $params   A hash containing any additional configuration or
     *                        connection parameters a subclass might need.
     *
     * @return mixed  The newly created concrete Accounts_Driver instance, or
     *                false on an error.
     */
    function &factory($driver = null, $params = null)
    {
        if (is_null($driver)) {
            $driver = $GLOBALS['conf']['server']['driver'];
        }

        if ($driver == 'null') {
            return $driver = &new Accounts_Driver();
        }

        if (is_null($params)) {
            $params = Horde::getDriverConfig('server', $driver);
        }

        include_once dirname(__FILE__) . '/Driver/' . $driver . '.php';
        $class = 'Accounts_Driver_' . $driver;
        if (class_exists($class)) {
            return $accounts = &new $class($params);
        } else {
            return false;
        }
    }

    /**
     * Attempts to return a reference to a concrete Accounts_Driver instance
     * based on $driver.
     *
     * It will only create a new instance if no Accounts_Driver instance with
     * the same parameters currently exists.
     * This should be used if multiple storage sources are required.
     * This method must be invoked as: $var = &Accounts_Driver::singleton()
     *
     * @param string $driver  Type of concrete Accounts_Driver subclass to
     *                        return.  The class is based on the accounts
     *                        driver ($driver).  The code is dynamically
     *                        included.
     *
     * @param array $params   A hash containing any additional configuration or
     *                        connection parameters a subclass might need.
     *
     * @return mixed  The created concrete Accounts_Driver instance, or false
     *                on error.
     */
    function &singleton($driver = null, $params = null)
    {
        static $instances;

        if (is_null($driver)) {
            $driver = $GLOBALS['conf']['server']['driver'];
        }

        if (is_null($params)) {
            $params = Horde::getDriverConfig('storage', $driver);
        }

        if (!isset($instances)) {
            $instances = array();
        }

        $signature = serialize(array($driver, $params));
        if (!isset($instances[$signature])) {
            $instances[$signature] = &Accounts_Driver::factory($driver, $params);
        }

        return $instances[$signature];
    }

    /**
     * Returns the username.
     *
     * @return string  The lowercased username.
     *
     */
    function getUsername()
    {
        $auth = explode('@', Auth::getAuth());
        return String::lower($auth[0]);
    }

    /**
     * Returns the user realm.
     *
     * @return string  The user realm.
     */
    function getRealm()
    {
        $auth = explode('@', Auth::getAuth());
        return isset($auth[1]) ? $auth[1] : '';
    }

    /**
     * Returns the user's quota if available.
     *
     * @return mixed  A quota array, elements are used bytes and limit bytes on
     *                success, false on failure.
     */
    function getQuota()
    {
        return false;
    }

    /**
     * Returns the user's full name.
     *
     * @return mixed  The user's full name (string), or false (error).
     */
    function getFullname()
    {
        return false;
    }

    /**
     * Returns the user's home (login) directory.
     *
     * @return mixed  The user's directory (string), or false (error).
     */
    function getHome()
    {
        return false;
    }

    /**
     * Returns the user's default shell.
     *
     * @return mixed  The user's shell (string), or false (error).
     */
    function getShell()
    {
        return false;
    }

    /**
     * Returns the date of the user's last password change.
     *
     * @return mixed  Date string (string) or false (error).
     */
    function getPasswordChange()
    {
        return false;
    }

    /**
     * Returns the status of the current password.
     *
     * @return mixed  A string with a warning message if the password is about
     *                to expire, PEAR_Error on error and false otherwise.
     */
    function checkPasswordStatus()
    {
        return false;
    }

}
