<?php
/**
 * $Horde: accounts/lib/Driver/localhost.php,v 1.17 2005/01/04 19:06:23 chuck Exp $
 *
 * Copyright 2002-2005 Eric Jon Rostetter <eric.rostetter@physics.utexas.edu>
 *
 * See the enclosed file LICENSE for license information (BSD). If you
 * did not receive this file, see http://www.horde.org/licenses/bsdl.php.
 *
 * Implements the Accounts API for IMAP servers with unix accounts on
 * the localhost machine (same machine as the web server).  Should work
 * for local unix accounts, nis/nis+ accounts, or any PAM oriented accounts
 * that appear as local accounts on the local machine.  The exception is
 * the quota support.  See that routine for additional comments.
 *
 * @author  Eric Rostetter <eric.rosttter@physics.utexas.edu>
 * @since   Accounts 2.1
 * @package accounts
 */
class Accounts_Driver_localhost extends Accounts_Driver {

    /**
     * Hash containing connection parameters.
     *
     * @var array $params
     */
    var $params = array();

    /**
     * Constructor.
     *
     * @access public
     *
     * @param array $params  Hash containing connection parameters.
     */
    function Accounts_Driver_localhost($params = array())
    {
        $this->params = $params;
        if (!isset($this->params['quota_path]')) {
            $this->params['quota_path'] = 'quota';
        }
        if (!isset($this->params['grep_path'])) {
            $this->params['grep_path'] = 'grep';
        }
    }

    /**
     * Returns the user account from the posix information.
     *
     * @return array  A hash with complete account details.
     */
    function _getAccount()
    {
        static $information;

        if (!isset($information)) {
            // this won't work if we don't have posix extensions
            if (!Util::extensionExists('posix')) {
                return PEAR::raiseError(_("POSIX extension is missing"));
            }

            $user = String::lower($this->getUsername());
            $information = posix_getpwnam($user);
        }

        return $information;
    }

    /**
     * Implement the Quota API for IMAP servers with a unix quota command.
     * This may require a modified "quota" command that allows the httpd server
     * account to get quotas for other users...  It requires that your
     * web server and imap server be the same server or at least have shared
     * authentication and file servers (e.g. via NIS/NFS).  And last, it (as
     * written) requires the posix php extensions.
     *
     * You must configure this in accounts/config/conf.php.  You must pass at
     * least the path to your quota and grep commands as parameters.  You may
     * also ask for mountpoint translations via the 'translateMountPoint'
     * boolean parameter, and if so you must specify the 'translationTable'
     * to use.  The translationTable is the name of a file to translate a
     * mount point to a device name.  Some common files for this are:
     * /etc/fstab, /etc/mtab, /etc/vfstab.  An example configuration might be
     * something like:
     *
     *   'quota' => array(
     *     'driver' => 'command',
     *     'params' => array(
     *       'translateMountPoint' => true,
     *       'translationTable' => '/etc/mtab',
     *       'quota_path' => '/usr/bin/quota',
     *       'grep_path'  => '/bin/grep'))
     *
     * Note that using the translateMountPoint option requires the File_Fstab
     * PEAR module to be installed on your system!
     *
     * If your quota command wraps the output onto two lines, then this module
     * will only work if you have a grep which supports the -A switch, and
     * you append an -A1 switch to your grep_path (e.g. '/bin/grep -A1').
     *
     * @access public
     *
     * @return mixed  The quota hash (bytes used, limit) or false for error.
     */
    function getQuota()
    {
        $information = $this->_getAccount();
        $homedir = $information['dir'];

        // If we want mount point translations, then translate the login dir
        // name to a mount point.  If not, then simply parse out the device
        // name from the login directory, and use that instead.
        if ($this->params['translateMountPoint'] &&
            file_exists($this->params['translationTable']))
        {
            require_once('File/Fstab.php');
            $sysTab = File_Fstab::singleton($this->params['translationTable']);
            do {
               $entry = &$sysTab->getEntryForPath($homedir);
               $homedir = dirname($homedir);
               if ($homedir == '.' || empty($homedir)) {
                   $homedir = '/';
               }
            } while (is_a($entry, 'PEAR_Error'));
            $mountPoint = $entry->device;
        } else {
           $homedir = explode('/', $homedir);
           $mountPoint = '/' . $homedir[1];
        }

        $cmdline = sprintf('%s -u %s 2>&1 | %s %s',
                           $this->params['quota_path'],
                           $this->getUserName(),
                           $this->params['grep_path'],
                           $mountPoint);
        $junk = exec($cmdline, $quota_data, $return_code);
        if ($return_code == 0 && !empty($quota_data[0])) {
           // In case of quota output wrapping on two lines, we concat
           // the second line of results, if any, here.
           if (!empty($quota_data[1])) {
               $quota_data[0] .= $quota_data[1];
           }
           // Now parse out the quota info and return it.
           $quota = preg_split('/\s+/', trim($quota_data[0]));
           return array('used' => $quota[1] * 1024, 'limit' => $quota[2] * 1024);
        }
        return false;
    }

    /**
     * Returns the user's full name.
     *
     * @return mixed  The user's full name (string), or false (error).
     */
    function getFullname()
    {
        $information = $this->_getAccount();
        $gecos_array = explode(',', $information['gecos']);
        return $gecos_array[0];
    }

    /**
     * Returns the user's home (login) directory.
     *
     * @return mixed  The user's directory (string), or false (error).
     */
    function getHome()
    {
        $information = $this->_getAccount();
        return $information['dir'];
    }

    /**
     * Returns the user's default shell.
     *
     * @return mixed  The user's shell (string), or false (error).
     */
    function getShell()
    {
        $information = $this->_getAccount();
        return $information['shell'];
    }

    /**
     * Returns the status of the current password.
     *
     * @return mixed  A string with a warning message if the password is about
     *                to expire, PEAR_Error on error and false otherwise.
     */
    function checkPasswordStatus()
    {
        $information = $this->_getAccount();
        if (is_a($information,'PEAR_Error')) {
            return $information;
        } else {
            return false;
        }
    }

}
